#include "pathnode.h"

#include <fstream.h>
#include <math.h>
#include <stdlib.h>
#include "pathedge.h"

int PathNode::numCalls = 0;

PathNode *PathNode::getNodePtr(const Point2& point, double maxPenFrac,
			       const Workspace *workspace) {
  PathNode *newNode;

  numCalls++;

  Frame2 newFrame;
  ftrans2(&newFrame, point);
  RobotState newState = workspace->testRobotState(&newFrame);
  if ((newState.dist/newState.obst->getMaxPen() < maxPenFrac) && 
      newState.inBound) {
    newNode = new PathNode(newFrame, newState, workspace);
    assert(newNode);
  } else {
    newNode = NULL;
  }

  return newNode;
}

PathNode::~PathNode() {
  while  (connections.length() > 0) {
    delete connections[0];
  }
  for (int i = 0; i < pNeighbors.length(); i++)
    pNeighbors[i]->removeNeighbor(this, false);
  for (int i = 0; i < sNeighbors.length(); i++)
    sNeighbors[i]->removeNeighbor(this, true);
}

void PathNode::findEdges(double maxPenFrac, PathPlanner *pathPlanner) {
  for (int i = 0; i < pNeighbors.length(); i++) {
      if (!connNodes.contains(pNeighbors[i]))
	PathEdge::makeEdgePtr(this, pNeighbors[i], maxPenFrac, 
			      pathPlanner, workspace);
  }
}

bool PathNode::moveToFree(double newPenFrac) {
  Frame2 newFrame;
  bool done = false;
  int numTries = 0;
  double offset = (newPenFrac == 0 ? 1 : .75);

  while (!done && numTries < 25) {
    double theta = drand48()*2*3.1415926;
    ftrans2(&newFrame, padd2(nodeFrame.p, 
			     pmul2(nodeState.dist*(offset+drand48()/3), 
				   point2(cos(theta), sin(theta)))));
    RobotState newState = workspace->testRobotState(&newFrame);
    if ((newState.dist/newState.obst->getMaxPen() < newPenFrac) && 
	newState.inBound) {
      this->nodeFrame = newFrame;
      this->nodeState = newState;
      done = true;
    }
    numTries++;
  }

  for (int i = 0; i < connections.length(); i++) {
    assert(connections[i]);
    connections[i]->moveNode();
  }

  return done;
}

void PathNode::saveNodesVect(DArray<PathNode *>& nodes, const char *outFileName) { 
  ofstream outfile(outFileName);
  assert(outfile);

  outfile << "VECT\n";
  outfile << nodes.length() << " " << nodes.length() << " 2\n";
  for (int i = 0; i < nodes.length(); i++)
    outfile << "1 ";
  outfile << "\n";
  outfile << "1 0 1 ";
  for (int i = 3; i < nodes.length(); i++)
    outfile << "0 ";
  outfile << "\n";
  for (int i = 0; i < nodes.length(); i++) {
    Point2 pos= nodes[i]->getPos();
    outfile << pos.x << " " << pos.y << " 0\n";
  }
  outfile << "0 0 1 1  0 0 0 1\n"; 

} 

void PathNode::addEdge(PathEdge *edge, PathNode *other) {
  assert(edge); 
  connections.append(edge);
  connNodes.append(other);
}

void PathNode::removeEdge(PathEdge *edge, PathNode *other) {
  connections.remove(edge);
  connNodes.remove(other);
}
