#include "pathplanner.h"

#include <assert.h>
#include <fstream.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include "distance/dist.h"
#include "pathedge.h"
#include "pathnode.h"
#include "util.h"

PathPlanner::PathPlanner(istream &workspaceIn, double myInitPen):
  workspace(workspaceIn), initPen(myInitPen) {}

PathPlanner::~PathPlanner() {
  for (int i = 0; i < pathNodes.length(); i++)
    delete pathNodes[i];
}

void PathPlanner::makePenNodes(int initNumNodes) {
  int nodesMade = 0;
  curPen = initPen;

  tStart = PathNode::getNodePtr(point2(.1, .1), 1, &workspace);
  tEnd = PathNode::getNodePtr(point2(.9, .9), 1, &workspace);
  assert(tStart);
  assert(tEnd);

  pathNodes.append(tStart);
  pathNodes.append(tEnd);

  while (nodesMade < initNumNodes) {
    PathNode *newNode = PathNode::getNodePtr(point2(drand48(), drand48()), 
					     1, &workspace);
    if (newNode) {
      pathNodes.append(newNode);
      nodesMade++;
    }
  }
}

void PathPlanner::findNeighbors(double proximity) {
  for (int i = 0; i < pathNodes.length(); i++)
    for (int j = i+1; j < pathNodes.length(); j++)
      if (pathNodes[i]->distToNode(pathNodes[j]) < proximity) {
	pathNodes[i]->addNeighbor(pathNodes[j], true);
	pathNodes[j]->addNeighbor(pathNodes[i], false);
      }
}
	

void PathPlanner::makePenPaths(double proximity) {
  findNeighbors(proximity);
  for (int i = 0; i < pathNodes.length(); i++)
    pathNodes[i]->findEdges(1, this);
}

void PathPlanner::regrowObstacles(double finalDepth) {
  double maxPenFrac = 1;

  DArray<PathNode *> origNodes(pathNodes);

  while (maxPenFrac > .01) {
    cout << "Regrowing..." << endl;
    maxPenFrac /= 4;
    if (maxPenFrac < finalDepth)
      maxPenFrac = 0;
    cout << "Starting with " << pathNodes.length() << " nodes, ";
    cout << pathEdges.length() << " edges." << endl;
    for (int i = 2; i < pathNodes.length(); i++) {
      if (pathNodes[i]->getDist() < maxPenFrac)
	if (!pathNodes[i]->moveToFree(maxPenFrac)) {
	  delete pathNodes[i];
	  pathNodes.removeAt(i--);
	  continue;
	}
      if (pathNodes[i]->numConnections() == 0) {
	delete pathNodes[i];
	pathNodes.removeAt(i--);
      }
    }
    int curPath = 0;
    int numPaths = pathEdges.length();
    while (curPath < numPaths) {
      if (!pathEdges[curPath]->freePath(maxPenFrac)) {
	pathEdges[curPath]->moveToFree(maxPenFrac);
	delete pathEdges[curPath];
	numPaths--;
      } else {
	curPath++;
      }
    }

    //for (int i = 0; i < pathNodes.length(); i++)
    //  pathNodes[i]->findEdges(maxPenFrac, this);
    cout << "Ending with " << pathNodes.length() << " nodes, ";
    cout << pathEdges.length() << " edges." << endl << endl;

  }
  cout << PathNode::getNumCalls() << endl;

  saveNodesVect("nodes_f.vect");

  for (int i = 0; i < origNodes.length(); i++)
    pathNodes.remove(origNodes[i]);

  saveNodesVect("nodes_a.vect");
}

bool PathPlanner::connected() {
  return PathEdge::connected(tStart, tEnd, pathEdges, pathNodes, false);
}

void PathPlanner::saveNodesVect(const char *outFileName) { 
  PathNode::saveNodesVect(pathNodes, outFileName);
} 

void PathPlanner::savePathsVect(const char *outFileName) { 
  PathEdge::saveEdgesVect(pathEdges, outFileName);
}
