#include "workspace.h"
#include "obstacle.h"
#include "util.h"
#include <string>
#include <assert.h>

Workspace::Workspace(istream& workData)
{
  ftrans2(&obstFrame, point2(0, 0));

  bgnsobj2();
  spoint2(point2(0, 0), 0, NULL);
  robotRep = endsobj2();

  readWorkData(workData);
  buildObstRep();
}

const RobotState Workspace::testRobotState(Frame2 *robotFrame) const
{
  RobotState state;

  Sdist2 dist = sdist2(robotRep, robotFrame, obstRep, &obstFrame, 0);
  state.obst = obstacles + *(int *)dist.sobj[1]->a;
  state.dist = (state.obst->pointInside(robotFrame->p)
		? -dist.d : dist.d);
  state.inBound = ((robotFrame->p.x > 0 && robotFrame->p.x < 1) &&
		   (robotFrame->p.y > 0 && robotFrame->p.y < 1));

  return state;
}

// Private
void Workspace::readWorkData(istream& workData)
{
  string fileType;
  int numEdges;

  workData >> fileType;
  assert(fileType == "OFF");
  workData >> numPts >> numObsts >> numEdges;

  points = new Point2[numPts];
  assert(points);
  for (int i = 0; i < numPts; i++) {
    workData >> points[i];
    if (verbose_input)
      cout << "Adding vertex " << i << ": x = " << points[i].x << " y = " <<
	points[i].y << endl;
  }

  obstacles = new Obstacle[numObsts];
  assert(obstacles);
  for (int i = 0; i < numObsts; i++)
    obstacles[i] = Obstacle(workData, this);
}

void Workspace::buildObstRep()
{
  bgnsobj2();
  for (int i = 0; i < numObsts; i++) {
    int *obstId = new int(i);
    assert(obstId);
    ssobj2(obstacles[i].getQuinRep(), &obstFrame, obstId);
  }
  obstRep = endsobj2();
}


